@@ -1,12 +1,15 @@
 import dataclasses
 import json
 from dataclasses import dataclass, field
-from typing import List, Union, Set, Dict, Any, Type
+from typing import List, Union, Set, Dict, Any, Type, Mapping
 
+from pydantic import Field
+from pydantic.fields import ModelField
 from typing_extensions import Literal
 
 from pycurd.const import QUERY_OP_COMPARE, QUERY_OP_RELATION, QUERY_OP_FROM_TXT
-from pycurd.error import UnknownQueryOperator, InvalidQueryConditionValue, InvalidQueryConditionColumn
+from pycurd.error import UnknownQueryOperator, InvalidQueryConditionValue, InvalidQueryConditionColumn, \
+    InvalidOrderSyntax
 from pycurd.types import RecordMapping, RecordMappingField
 
 
@@ -76,7 +79,7 @@ class QueryField:
 @dataclass
 class QueryOrder:
     column: RecordMappingField
-    order: Union[Literal['asc', 'desc']]
+    order: Union[Literal['asc', 'desc', 'default']]
 
     def __eq__(self, other):
         if isinstance(other, QueryOrder):
@@ -86,6 +89,37 @@ class QueryOrder:
     def __repr__(self):
         return '<QueryOrder %r.%s>' % (self.column, self.order)
 
+    @classmethod
+    def from_text(cls, table: Type[RecordMapping], text):
+        """
+        :param text: order=id.desc, xxx.asc
+        :return: [
+            [<column>, asc|desc|default],
+            [<column2>, asc|desc|default],
+        ]
+        """
+        orders = []
+        for i in map(str.strip, text.split(',')):
+            items = i.split('.', 2)
+
+            if len(items) == 1:
+                column_name, order = items[0], 'default'
+            elif len(items) == 2:
+                column_name, order = items
+            else:
+                raise InvalidOrderSyntax("Invalid order syntax")
+
+            column = getattr(table, column_name, None)
+            if column is None:
+                raise InvalidOrderSyntax('Unknown column: %s' % column_name)
+
+            order = order.lower()
+            if order not in ('asc', 'desc', 'default'):
+                raise InvalidOrderSyntax('Invalid order mode: %s' % order)
+
+            orders.append(cls(column, order))
+        return orders
+
 
 @dataclass
 class ConditionExpr:
@@ -214,7 +248,7 @@ class QueryInfo:
     select_exclude: Set[Union[RecordMappingField, Any]] = field(default_factory=lambda: set())
 
     conditions: QueryConditions = None
-    order_by: List = field(default_factory=lambda: [])
+    order_by: List[QueryOrder] = field(default_factory=lambda: [])
 
     foreign_keys: Dict[str, 'QueryInfo'] = None
 
@@ -249,7 +283,7 @@ class QueryInfo:
         )
 
     @classmethod
-    def from_json(cls, table, data, from_http_query=False):
+    def from_json(cls, table: Type[RecordMapping], data, from_http_query=False, check_cond_with_field=False):
         assert table, 'table must be exists'
         get_items = lambda keys: [getattr(table, x) for x in keys]
         q = cls(table)
@@ -278,12 +312,31 @@ class QueryInfo:
 
             return selected, unselected
 
-        def parse_value(val):
-            if isinstance(val, str) and val.startswith('$'):
-                a, b = val.split(':', 1)
-                t = RecordMapping.all_mappings.get(a[1:])
-                return getattr(t, b)
-            return val
+        def parse_value(_key, field_name, value, *, is_list=False):
+            value = http_value_try_parse(value)
+
+            if check_cond_with_field:
+                if isinstance(value, str) and value.startswith('$'):
+                    a, b = value.split(':', 1)
+                    t = RecordMapping.all_mappings.get(a[1:])
+                    return getattr(t, b)
+
+            model_field = table.__fields__.get(field_name)
+
+            if is_list:
+                assert isinstance(value, List), 'The right value of relation operator must be list'
+                final_value = []
+                for i in value:
+                    val, err = model_field.validate(i, None, loc=_key)
+                    if err:
+                        raise InvalidQueryConditionValue('invalid value: %s' % value)
+                    final_value.append(val)
+            else:
+                final_value, err = model_field.validate(value, None, loc=_key)
+                if err:
+                    raise InvalidQueryConditionValue('invalid value: %s' % value)
+
+            return final_value
 
         def parse_conditions(data):
             conditions = []
@@ -304,18 +357,16 @@ class QueryInfo:
 
                 elif '.' in key:
                     field_name, op_name = key.split('.', 1)
-                    value = http_value_try_parse(value)
 
                     op = QUERY_OP_FROM_TXT.get(op_name)
                     if op is None:
                         raise UnknownQueryOperator(op_name)
 
-                    if op in (QUERY_OP_RELATION.IN, QUERY_OP_RELATION.NOT_IN, QUERY_OP_RELATION.CONTAINS):
-                        value = http_value_try_parse(value)
-                        assert isinstance(value, List), 'The right value of relation operator must be list'
+                    is_list = op in (QUERY_OP_RELATION.IN, QUERY_OP_RELATION.NOT_IN, QUERY_OP_RELATION.CONTAINS)
 
                     try:
                         field_ = getattr(table, field_name)
+                        value = parse_value(key, field_name, value, is_list=is_list)
                         conditions.append(ConditionExpr(field_, op, value))
                     except AttributeError:
                         raise InvalidQueryConditionColumn("column not exists: %s" % field_name)
@@ -328,9 +379,18 @@ class QueryInfo:
         for key, value in data.items():
             if key.startswith('$'):
                 if key == '$order-by':
-                    pass
-                elif key == '$foreign-key':
-                    pass
+                    q.order_by = QueryOrder.from_text(table, value)
+                elif key == '$fks' or key == '$foreign-keys':
+                    value = http_value_try_parse(value)
+                    assert isinstance(value, Mapping)
+                    q.foreign_keys = {}
+
+                    for k, v in value.items():
+                        k2 = k[:-2] if k.endswith('[]') else k
+                        t = table.all_mappings.get(k2)
+
+                        if t:
+                            q.foreign_keys[k] = cls.from_json(t, v, check_cond_with_field)
 
                 continue
 
